<?php
/*
 * Omnistream Sidebar Ad Widget Class
*/

// Block direct requests
if (!defined('ABSPATH'))
  die('-1');

/**
 * Adds Omnistream_Sidebar_Ad widget.
 */
class Omnistream_Sidebar_Ad extends WP_Widget {
  /**
   * Register widget with WordPress.
   */
  function __construct() {
    parent::__construct(
      'omnistream_sidebar-ad', // Base ID
      __('Omnistream Sidebar Ad Widget', 'text_domain'), // Name
      array('description' => __('Sidebar Ads via Omnistream', 'text_domain'),) // Args
    );
  }
  /**
   * Front-end display of widget.
   *
   * @see WP_Widget::widget()
   *
   * @param array $args     Widget arguments.
   * @param array $instance Saved values from database.
   *
   * This function finds an ad at any arbitrary level in the category hierarchy.
   * The code will check for an ad that matches the category on the page, or any parent category.
   * Wordpress naturally includes all categories below in a post query, so you have
   * to exclude ads of any sub-category, and in the case of the home page, you have
   * exclude all categories.
   */
  public function widget($args, $instance) {
    global $post;

    // Set default values if not set yet
    $instance = wp_parse_args((array) $instance, $this->sidebar_ad_defaults($instance));

    // assume we are on the home page, and get default and home ad category
    $ad_category_ID_list = $instance['home_category'];
    $ad_category = get_category($instance['home_category']);

    if (is_front_page()) {
      if ($instance['homepage_ads'] != 'on') {
        return;    // don't show an ad on the home page
      }
    } elseif (is_single()) {
      $categories = get_the_category(); // get category lists to search
      // a post can be in several categories, so let's search for an ad in any of them
      if (!empty($categories)) {
        $ad_category = $categories[0]; // save the first in case we need to look at parent
        $ad_category_ID_list = '';
        foreach ($categories as $cat) {
          $ad_category_ID_list .= $cat->cat_ID . ',';
        }
        $ad_category_ID_list = rtrim($ad_category_ID_list, ',');
      }
    }
    else { // not front page, and not single, so we are on a category page
      $categories = get_the_category();
      if (empty($categories)) {
        $ad_category = get_category_by_slug($post->post_name);
        if (!$ad_category) {
          return; // we have no idea where were are, so don't show an ad
        }
      } else {
        $ad_category = $categories[0];
      }
      // we don't want to include ads from sub-categories
      $excluded_cats = '';
      $child_cats = get_categories(array('child_of' => $ad_category->cat_ID));
      if (count($child_cats) > 0) {
        foreach ($child_cats as $cat) {
          $excluded_cats .= ',-' . $cat->term_id;
        }
      }
      $ad_category_ID_list = $ad_category->cat_ID . $excluded_cats;
    }

    // Look for the latest current sidebar ad
    do {
      $q_args = array(
        'post_type' => 'omnistream-ad',
        'category' => $ad_category_ID_list,
        'meta_key' => 'OmnistreamAdPosition',
        'meta_value'=> 'sidebar',
        'meta_query' => array(
          'relation' => 'OR',
          array(
            'key' => 'OmnistreamPostExpiry',
            'value' => gmdate("Y-m-d\TH:i:s\Z"),
            'compare' => '>'
          ),
          array(
            'key' => 'OmnistreamPostExpiry',
            'value' => '',
            'compare' => 'NOT EXISTS'
          )
        ),
        'orderby' => 'date',
        'order'   => 'DESC',
        'post_status' => 'publish',
        'posts_per_page' => 1
      );
      $latestAds = get_posts($q_args);

      if (count($latestAds) > 0) {
        break;
      } else { // look for ad in parent category
        $temp_parent_category_ID = $ad_category->category_parent;
        if ($temp_parent_category_ID === 0) {
          break;
        }
        $ad_category = get_category($temp_parent_category_ID);
        $ad_category_ID_list = $ad_category->cat_ID . get_sub_categories_for_exclusion($ad_category->cat_ID);
      }
    } while (1);

    if (count($latestAds) > 0) {
      $ad = $latestAds[0];

      $q_args = array(
        'post_type' => 'attachment',
        'post_parent' => $ad->ID,
        'posts_per_page' => -1
      );
      $ad_images = get_posts($q_args);

      $ad_sizes = array();
      foreach ($ad_images as $ad_image) {
        $ad_sizes[] = get_post_meta($ad_image->ID, 'OmnistreamAdSize', true);
      }

      $adAlt = get_post_meta($ad->ID, 'OmnistreamAdAlt', true);
      $adURL = get_post_meta($ad->ID, 'OmnistreamAdURL', true);

      extract($args);

      echo $before_widget;
      echo '<div class="ad-sidebar">';

      $i = array_search('mediumRectangle', $ad_sizes);
      $j = array_search('halfPage', $ad_sizes);
      $k = array_search('portrait', $ad_sizes);
      if (isset($i)) {
        $attachment_url = wp_get_attachment_url($ad_images[$i]->ID);
        echo <<<WIDGET_MR
          <div class="medium-rectangle">
            <a href="$adURL" target="_blank" data-post-guid="$ad->guid" data-event="ad-click">
            <img alt="$adAlt" src="$attachment_url" width="100%">
            </a>
          </div>
WIDGET_MR;
      } elseif (isset($j)) {
        $attachment_url = wp_get_attachment_url($ad_images[$j]->ID);
        echo <<<WIDGET_HP
          <div class="half-page">
            <a href="$adURL" target="_blank" data-post-guid="$ad->guid" data-event="ad-click">
            <img alt="$adAlt" src="$attachment_url" width="100%">
            </a>
          </div>
WIDGET_HP;
      } elseif (isset($k)) {
        $attachment_url = wp_get_attachment_url($ad_images[$k]->ID);
        echo <<<WIDGET_P
          <div class="portrait">
            <a href="$adURL" target="_blank" data-post-guid="$ad->guid" data-event="ad-click">
            <img alt="$adAlt" src="$attachment_url" width="100%">
            </a>
          </div>
WIDGET_P;
      }
      echo '</div>';
      echo $after_widget;
    }
  }
  /**
   * Back-end widget form.
   *
   * @see WP_Widget::form()
   *
   * @param array $instance Previously saved values from database.
   */
  public function form($instance) {

    // Set default values if not set yet
    $instance = wp_parse_args((array) $instance, $this->sidebar_ad_defaults($instance));

    $homepage_ads = esc_attr($instance['homepage_ads']);
    $home_category = esc_attr($instance['home_category']);

    ?>
    <p>
      <input class="checkbox" type="checkbox" <?php checked($instance['homepage_ads'], "on") ?> id="<?php echo $this->get_field_id('homepage_ads'); ?>" name="<?php echo $this->get_field_name('homepage_ads'); ?>" />
      <label for="<?php echo $this->get_field_id('homepage_ads'); ?>"><?php _e('Show ads on home page'); ?></label>
    </p>
    <p>
      <label for="<?php echo $this->get_field_id('home_category'); ?>"><?php _e('Ad category for home page:'); ?></label>
      <?php
        $included_cats = '';
        $top_level_cats = get_categories(array('parent' => 0));
        if (count($top_level_cats) > 0) {
          foreach ($top_level_cats as $cat) {
            $included_cats .= $cat->cat_ID . ',';
          }
          $included_cats = rtrim($included_cats, ',');
        }

        $args = array(
          'name'       => $this->get_field_name('home_category'),
          'id'         => $this->get_field_id('home_category'),
          'selected'   => $home_category,
          'include'    => $included_cats,
          'orderby'    => 'name',
          'hide_empty' => false
        );
        wp_dropdown_categories($args);
      ?>
    </p>
    <?php
  }
  /**
   * Sanitize widget form values as they are saved.
   *
   * @see WP_Widget::update()
   *
   * @param array $new_instance Values just sent to be saved.
   * @param array $old_instance Previously saved values from database.
   *
   * @return array Updated safe values to be saved.
   */
  public function update($new_instance, $old_instance) {
    $instance = array();
    $instance['homepage_ads'] = (! empty($new_instance['homepage_ads'])) ? strip_tags($new_instance['homepage_ads']) : '';
    $instance['home_category'] = (! empty($new_instance['home_category'])) ? strip_tags($new_instance['home_category']) : '';
    return $instance;
  }

  function sidebar_ad_defaults($instance) {
    if (!isset($instance['home_category'])) {
      $site_categories = get_categories(array('orderby' => 'term_id'));
      $default_category = $site_categories[0]->cat_ID;  // use first as default
    } else {
      $default_category = $instance['home_category'];
    }

    return array(
      'homepage_ads' => 'false',
      'home_category' => $default_category
    );
  }
} // class My_Widget

add_action('widgets_init', function(){
     register_widget('Omnistream_Sidebar_Ad');
});
